/*
 * Copyright (c) 2006-2010 Maskat Project.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Maskat Project - initial API and implementation
 */
package jp.sf.maskat.core.layout;

import java.util.ArrayList;
import java.util.List;

import jp.sf.maskat.core.MaskatElement;

import org.apache.commons.beanutils.BasicDynaClass;
import org.apache.commons.beanutils.DynaProperty;

/**
 * UnknownComponentクラスのjava.lang.Classの機能をシミュレーションしたクラスです
 * 
 *  Classの機能の他に以下の情報を保持します
 *  
 * ・完全修飾ドメイン名
 */
public class UnknownComponentClass extends BasicDynaClass {

	/**
	 * シリアルバージョンUID
	 */
    private static final long serialVersionUID = 6649622981353389752L;

	/**
	 * この部品の完全修飾ドメイン名
	 */
    private String qualifiedName;

    /**
     * デフォルトコンストラクタです
     *
     */
    public UnknownComponentClass() {
        super();
    }

    /**
     * コンストラクタです
     * 
     * @param name クラス名
     * @param dynaBeanClass DynaBeanClassクラス
     * @param properties プロパティ情報
     */
    public UnknownComponentClass(String name, Class dynaBeanClass,
            DynaProperty[] properties) {
        super(name, dynaBeanClass, properties);
        setqualifiedName(name);
    }

    /**
     * コンストラクタです
     * 
     * @param name クラス名
     * @param dynaBeanClass DynaBeanClassクラス
     * @param prefix 修飾子
     * @param properties プロパティ情報
     */
    public UnknownComponentClass(String name, Class dynaBeanClass,
            String prefix, String[] properties) {
        super(name, dynaBeanClass, null);
        boolean isns = prefix != null && prefix.trim().length() > 0;
        String qName = isns ?
        		(new StringBuffer(prefix))
        			.append(":").append(name).toString() : name;
        setqualifiedName(qName);
        setProperties(properties);
    }

    /**
     * プロパティ情報リストをこのDynaBeanClassに登録します
     * 
     * @param properties プロパティ情報
     */
    private void setProperties(String[] properties) {
   		List list = new ArrayList();
   		list.add(new DynaProperty("parent", MaskatElement.class));
   		list.add(new DynaProperty("children", List.class));
   		for (int i=0; properties != null && i < properties.length; i++) {
    		if ("context".equals(properties[i])) {
    			list.add(new jp.sf.maskat.core.layout.DynaProperty("context", String.class, false));
    		} else {
    			list.add(new DynaProperty(properties[i], String.class));
    		}
    	}
   		setProperties((DynaProperty[]) list.toArray(new DynaProperty[list.size()]));
   	}

    /**
     * {@inheritDoc}
     */
    public void setDynaProperties(DynaProperty[] properties) {
        setProperties(properties);
    }

    /**
     * {@inheritDoc}
     */
    public void setDynaBeanClass(Class dynaBeanClass) {
        super.setDynaBeanClass(dynaBeanClass);
    }

	/**
	 * この部品の完全修飾ドメイン名を取得します
	 * 
	 * @return 完全修飾ドメイン名
	 */
    public String getQualifiedName() {
        return this.qualifiedName;
    }

	/**
	 * この部品の完全修飾ドメイン名を設定します
	 * 
	 * @param name 完全修飾ドメイン名
	 */
    public void setqualifiedName(String name) {
        this.qualifiedName = name;
    }
}
