/*
 * Copyright (c) 2006-2010 Maskat Project.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Maskat Project - initial API and implementation
 */
package jp.sf.maskat.ui;

import jp.sf.maskat.ui.wizards.application.NewMaskatApplicationWizard;
import jp.sf.maskat.ui.wizards.layout.NewLayoutFileWizard;
import jp.sf.maskat.ui.wizards.project.NewMaskatProjectWizard;

import org.eclipse.ui.IFolderLayout;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IPerspectiveFactory;

/**
 * マスカットパースペクティブを構成するファクトリクラスです
 * 
 *  マスカットパースペクティブは以下の構成で作成されます
 *  
 *  ・画面構成
 *    フォルダ領域 幅20%, 高さ50%
 *    アウトライン領域 幅20%, 高さ50%
 *    エディタ領域 幅80%, 高さ66% 
 *    プロパティシート領域  幅80%, 高さ34%
 *  ・ウィザード
 *    新規マスカットプロジェクト作成
 *    新規マスカットアプリケーション作成
 *    新規レイアウト定義XML作成
 */
public class MaskatPerspectiveFactory implements IPerspectiveFactory {

	/**
	 * パレットViewID
	 */
	public static final String ID_PALETTE = "org.eclipse.gef.ui.palette_view"; //$NON-NLS-1$

	/**
	 * プロジェクトエクスプローラViewID
	 */
	public static final String ID_PROJECT_EXPLORER = "org.eclipse.ui.navigator.ProjectExplorer"; //$NON-NLS-1$

    /**
     * {@inheritDoc}
     */
	public void createInitialLayout(IPageLayout layout) {
		defineActions(layout);
		defineLayout(layout);
	}

	/**
	 * このパースペクティブにショートカットを定義します
	 * 
	 * @param layout レイアウト
	 */
	private void defineActions(IPageLayout layout) {
		layout.addNewWizardShortcut(NewMaskatProjectWizard.WIZARD_ID);
		layout.addNewWizardShortcut(NewLayoutFileWizard.WIZARD_ID);
		layout.addNewWizardShortcut(NewMaskatApplicationWizard.WIZARD_ID);

		layout.addShowViewShortcut(ID_PROJECT_EXPLORER);
		layout.addShowViewShortcut(IPageLayout.ID_OUTLINE);
		layout.addShowViewShortcut(IPageLayout.ID_PROP_SHEET);
		layout.addShowViewShortcut(ID_PALETTE);
	}

	/**
	 * このパースペクティブにレイアウトを定義します
	 * 
	 * @param layout レイアウト
	 */
	private void defineLayout(IPageLayout layout) {
		String editorArea = layout.getEditorArea();

		IFolderLayout topLeft = layout.createFolder(
				"topLeft", IPageLayout.LEFT, //$NON-NLS-1$ 
				(float) 0.20, editorArea);

		topLeft.addView(ID_PROJECT_EXPLORER);

		topLeft.addPlaceholder(ID_PALETTE);

		IFolderLayout bottomLeft = layout.createFolder(
				"bottomLeft", IPageLayout.BOTTOM, (float) 0.50, //$NON-NLS-1$
				"topLeft"); //$NON-NLS-1$
		bottomLeft.addView(IPageLayout.ID_OUTLINE);

		IFolderLayout bottomRight = layout.createFolder(
				"bottomRight", IPageLayout.BOTTOM, (float) 0.66, //$NON-NLS-1$
				editorArea);
		bottomRight.addView(IPageLayout.ID_PROP_SHEET);
		bottomRight.addPlaceholder(IPageLayout.ID_TASK_LIST);
		bottomRight.addPlaceholder(IPageLayout.ID_BOOKMARKS);
	}
}
